@@ -444,11 +444,46 @@ public:
    * \brief Access function to receive the latest data package sent from the robot through RTDE
    * interface.
    *
+   * \deprecated This method allocates memory on each call. Please use the overload which takes a
+   * reference to an existing DataPackage instead. This function will be removed in May 2027.
+   *
    * \returns The latest data package on success, a nullptr if no package can be found inside a preconfigured time
    * window.
    */
+  [[deprecated("This method allocates memory on each call. Please use the overload which takes a reference to an "
+               "existing DataPackage instead. This function will be removed in May 2027.")]]
   std::unique_ptr<rtde_interface::DataPackage> getDataPackage();
 
+  /*!
+   * \brief Return the latest RTDE data package received
+   *
+   * When packages are read from the background thread, the latest data package
+   * received from the robot can be fetched with this.    *
+   * When packages are not read from the background thread, this function will return false and
+   * print an error message.
+   *
+   * \param data_package Reference to a DataPackage where the received data package will be stored
+   * if a package was fetched successfully.
+   *
+   * \returns Whether a data package was received successfully
+   */
+  bool getDataPackage(rtde_interface::DataPackage& data_package);
+
+  /*!
+   * \brief Blocking call to get the next data package received from the robot.
+   *
+   * This function will block until a new data package is received from the robot and return it.
+   *
+   * \param data_package Reference to a unique ptr where the received data package will be stored.
+   * For optimal performance, the data package pointer should contain a pre-allocated data package
+   * that was initialized with the same output recipe as used in this RTDEClient. If it is not an
+   * initialized data package, a new one will be allocated internally which will have a negative
+   * performance impact and print a warning.
+   *
+   * \returns Whether a data package was received successfully
+   */
+  bool getDataPackageBlocking(std::unique_ptr<rtde_interface::DataPackage>& data_package);
+
   uint32_t getControlFrequency() const
   {
     return rtde_client_->getTargetFrequency();
@@ -728,6 +763,13 @@ public:
    */
   bool endToolContact();
 
+  /*!
+   * \brief Set the TCP offset of the robot.
+   *
+   * \returns True, if the write was performed successfully, false otherwise.
+   */
+  bool setTcpOffset(const vector6d_t& tcp_offset);
+
   /*!
    * \brief Set friction compensation for the torque_command. If true the torque command will compensate for friction,
    * if false it will not.
@@ -778,8 +820,12 @@ public:
    *
    * After initialization, the cyclic RTDE communication is not started automatically, so that data
    * consumers can be started also at a later point.
+   *
+   * \param read_packages_in_background If true, RTDE packages will be read in a background thread.
+   * In this case use getDataPackage() to receive the latest package. If set to false,
+   * getDataPackageBlocking() has to be called frequently.
    */
-  void startRTDECommunication();
+  void startRTDECommunication(const bool read_packages_in_background = true);
 
   /*!
    * \brief Sends a stop command to the socket interface which will signal the program running on
@@ -1004,6 +1050,26 @@ public:
     return primary_client_;
   }
 
+  /*! \brief Enable or disable background reading of RTDE packages.
+   *
+   * When enabled, RTDE packages will be read in a background thread.
+   * In this case use getDataPackage() to receive the latest package. If set to false,
+   * getDataPackageBlocking() has to be called frequently if RTDE communication is active.
+   *
+   * \param enabled Whether background reading should be enabled or disabled.
+   */
+  void setRTDEBackgroundReadEnabled(const bool enabled)
+  {
+    if (enabled)
+    {
+      rtde_client_->startBackgroundRead();
+    }
+    else
+    {
+      rtde_client_->stopBackgroundRead();
+    }
+  }
+
 private:
   void init(const UrDriverConfiguration& config);
 
@@ -1044,4 +1110,4 @@ private:
   VersionInformation robot_version_;
 };
 }  // namespace urcl
-#endif  // ifndef UR_CLIENT_LIBRARY_UR_UR_DRIVER_H_INCLUDED
\ No newline at end of file
+#endif  // ifndef UR_CLIENT_LIBRARY_UR_UR_DRIVER_H_INCLUDED
