@@ -54,7 +54,7 @@ help()
   echo "    -n             Name of the docker container. Defaults to '$CONTAINER_NAME'"
   echo "    -i             IP address the container should get. Defaults to $IP_ADDRESS"
   echo "    -d             Detached mode - start in background"
-  echo "    -f             Specify port forwarding to use. Defaults to '$PORT_FORWARDING'. Set to empty string to disable port forwarding."
+  echo "    -f             Specify port forwarding to use. Defaults to '$PORT_FORWARDING'. Set to \"DISABLED\" to disable port forwarding."
   echo "    -h             Print this Help."
   echo
 }
@@ -419,6 +419,7 @@ main() {
 
   URCAP_STORAGE="${PERSISTENT_BASE}/${ROBOT_SERIES}/urcaps"
   PROGRAM_STORAGE="${PERSISTENT_BASE}/${ROBOT_SERIES}/${ROBOT_MODEL}/programs"
+  POLYSCOPE_STORAGE="${PERSISTENT_BASE}/${ROBOT_SERIES}/${ROBOT_MODEL}/polyscope"
 
   strip_robot_model "$ROBOT_MODEL" "$ROBOT_SERIES"
 
@@ -430,6 +431,11 @@ main() {
     fi
   fi
 
+  if [ "$PORT_FORWARDING" == "DISABLED" ]; then
+    echo "Port forwarding disabled"
+    PORT_FORWARDING=""
+  fi
+
   DOCKER_ARGS=""
 
   if [ "$ROBOT_SERIES" == "polyscopex" ]; then
@@ -481,6 +487,7 @@ main() {
     docker_cmd="docker run --rm -d --net ursim_net --ip $IP_ADDRESS\
       -v ${URCAP_STORAGE}:/urcaps \
       -v ${PROGRAM_STORAGE}:/ursim/programs \
+      -v ${POLYSCOPE_STORAGE}:/ursim/.polyscope \
       -e ROBOT_MODEL=${ROBOT_MODEL} \
       $PORT_FORWARDING \
       --name $CONTAINER_NAME \
