@@ -188,6 +188,8 @@ void UrDriver::init(const UrDriverConfiguration& config)
   URCL_LOG_DEBUG("Initialization done");
 }
 
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wdeprecated-declarations"
 std::unique_ptr<rtde_interface::DataPackage> urcl::UrDriver::getDataPackage()
 {
   // This can take one of two values, 0ms or 100ms. The large timeout is for when the robot is commanding the control
@@ -197,6 +199,21 @@ std::unique_ptr<rtde_interface::DataPackage> urcl::UrDriver::getDataPackage()
 
   return rtde_client_->getDataPackage(timeout);
 }
+#pragma GCC diagnostic pop
+
+bool UrDriver::getDataPackage(rtde_interface::DataPackage& data_package)
+{
+  // This can take one of two values, 0ms or 100ms. The large timeout is for when the robot is commanding the control
+  // loop's timing (read is blocking). The zero timeout is for when the robot is sharing a control loop with
+  // something else (combined_robot_hw)
+  std::chrono::milliseconds timeout(get_packet_timeout_);
+  return rtde_client_->getDataPackage(data_package, timeout);
+}
+
+bool UrDriver::getDataPackageBlocking(std::unique_ptr<rtde_interface::DataPackage>& data_package)
+{
+  return rtde_client_->getDataPackageBlocking(data_package);
+}
 
 bool UrDriver::writeJointCommand(const vector6d_t& values, const comm::ControlMode control_mode,
                                  const RobotReceiveTimeout& robot_receive_timeout)
@@ -298,6 +315,26 @@ bool UrDriver::setPayload(const float mass, const vector3d_t& cog)
   }
 }
 
+bool UrDriver::setGravity(const vector3d_t& gravity)
+{
+  if (script_command_interface_->clientConnected())
+  {
+    return script_command_interface_->setGravity(&gravity);
+  }
+  else
+  {
+    URCL_LOG_WARN("Script command interface is not running. Falling back to sending plain script code. On e-Series "
+                  "robots this will only work, if the robot is in remote_control mode.");
+    std::stringstream cmd;
+    cmd.imbue(std::locale::classic());  // Make sure, decimal divider is actually '.'
+    cmd << "sec setup():" << std::endl
+        << " set_gravity([" << gravity[0] << ", " << gravity[1] << ", " << gravity[2] << "])" << std::endl
+        << "end";
+    return sendScript(cmd.str());
+    return true;
+  }
+}
+
 bool UrDriver::setToolVoltage(const ToolVoltage voltage)
 {
   // Test that the tool voltage is either 0, 12 or 24.
@@ -554,15 +591,28 @@ bool UrDriver::ftRtdeInputEnable(const bool enabled, const double sensor_mass,
   }
 }
 
+bool UrDriver::setTcpOffset(const vector6d_t& tcp_offset)
+{
+  if (script_command_interface_->clientConnected())
+  {
+    return script_command_interface_->setTcpOffset(tcp_offset);
+  }
+  else
+  {
+    URCL_LOG_ERROR("Script command interface is not running. Unable to set TCP offset.");
+    return false;
+  }
+}
+
 bool UrDriver::writeKeepalive(const RobotReceiveTimeout& robot_receive_timeout)
 {
   vector6d_t* fake = nullptr;
   return reverse_interface_->write(fake, comm::ControlMode::MODE_IDLE, robot_receive_timeout);
 }
 
-void UrDriver::startRTDECommunication()
+void UrDriver::startRTDECommunication(const bool read_packages_in_background)
 {
-  rtde_client_->start();
+  rtde_client_->start(read_packages_in_background);
 }
 
 bool UrDriver::stopControl()
