@@ -29,33 +29,40 @@
 #include <algorithm>
 #include <gtest/gtest.h>
 #include <algorithm>
+#include <chrono>
 #include <cmath>
-#include <optional>
+#include <thread>
 #include <unordered_map>
 #include <utility>
 #include <iostream>
+#include "ur_client_library/comm/tcp_server.h"
 #include "ur_client_library/exceptions.h"
 
 #include <ur_client_library/rtde/rtde_client.h>
 #include <ur_client_library/ur/version_information.h>
 
+#include "fake_rtde_server.h"
+#include "ur_client_library/helpers.h"
+
 using namespace urcl;
 
 std::string g_ROBOT_IP = "192.168.56.101";
+uint32_t g_FAKE_RTDE_PORT = 13875;
 
 class RTDEClientTest : public ::testing::Test
 {
 protected:
   void SetUp()
   {
-    client_.reset(new rtde_interface::RTDEClient(g_ROBOT_IP, notifier_, output_recipe_file_, input_recipe_file_));
+    client_.reset(
+        new rtde_interface::RTDEClient(g_ROBOT_IP, notifier_, resources_output_recipe_, resources_input_recipe_));
   }
 
   void TearDown()
   {
     client_.reset();
     // If we don't sleep we can get a conflict between two tests controlling the same rtde inputs
-    std::this_thread::sleep_for(std::chrono::seconds(1));
+    std::this_thread::sleep_for(std::chrono::milliseconds(100));
   }
 
   std::string output_recipe_file_ = "resources/rtde_output_recipe.txt";
@@ -172,8 +179,12 @@ TEST_F(RTDEClientTest, unconfigured_target_frequency)
 
 TEST_F(RTDEClientTest, set_target_frequency)
 {
-  client_.reset(
-      new rtde_interface::RTDEClient(g_ROBOT_IP, notifier_, output_recipe_file_, input_recipe_file_, 1, false));
+  // Set a target frequency that is different from the maximum frequency but a factor of it.
+  // Since we check timestamp differences, we need to make sure that the target frequency is
+  // achievable. 25 Hz is a factor of both 125 Hz and 500 Hz.
+  const double target_frequency = 25.0;
+  client_.reset(new rtde_interface::RTDEClient(g_ROBOT_IP, notifier_, output_recipe_file_, input_recipe_file_,
+                                               target_frequency, false));
   client_->init();
 
   // Maximum frequency should still be equal to the robot's maximum frequency
@@ -188,35 +199,23 @@ TEST_F(RTDEClientTest, set_target_frequency)
     EXPECT_EQ(client_->getMaxFrequency(), expected_max_frequency);
   }
 
-  double expected_target_frequency = 1;
-  EXPECT_EQ(client_->getTargetFrequency(), expected_target_frequency);
+  EXPECT_EQ(client_->getTargetFrequency(), target_frequency);
 
-  EXPECT_TRUE(client_->start());
+  EXPECT_TRUE(client_->start(false));
 
-  // Test that we receive packages with a frequency of 2 Hz
-  const std::chrono::milliseconds read_timeout{ 10000 };
-  std::unique_ptr<rtde_interface::DataPackage> data_pkg = client_->getDataPackage(read_timeout);
-  if (data_pkg == nullptr)
-  {
-    std::cout << "Failed to get data package from robot" << std::endl;
-    GTEST_FAIL();
-  }
+  // Test that we receive packages with a frequency of `target_frequency` Hz
+  auto data_pkg = std::make_unique<rtde_interface::DataPackage>(client_->getOutputRecipe());
+  ASSERT_TRUE(client_->getDataPackageBlocking(data_pkg));
 
   double first_time_stamp = 0.0;
   data_pkg->getData("timestamp", first_time_stamp);
 
-  data_pkg = client_->getDataPackage(read_timeout);
-  if (data_pkg == nullptr)
-  {
-    std::cout << "Failed to get data package from robot" << std::endl;
-    GTEST_FAIL();
-  }
-
+  ASSERT_TRUE(client_->getDataPackageBlocking(data_pkg));
   double second_time_stamp = 0.0;
   data_pkg->getData("timestamp", second_time_stamp);
 
-  // There should be 1 second between each timestamp
-  EXPECT_NEAR(second_time_stamp - first_time_stamp, 1, 1e-6);
+  // There should be 0.1 second between each timestamp
+  EXPECT_NEAR(second_time_stamp - first_time_stamp, 1.0 / target_frequency, 1e-6);
 
   client_->pause();
 }
@@ -271,7 +270,17 @@ TEST_F(RTDEClientTest, output_recipe_file)
   }
 }
 
-TEST_F(RTDEClientTest, recipe_compairson)
+TEST_F(RTDEClientTest, input_recipe_with_invalid_key)
+{
+  std::vector<std::string> actual_input_recipe = resources_input_recipe_;
+  actual_input_recipe.push_back("i_do_not_exist");
+
+  EXPECT_THROW(client_.reset(new rtde_interface::RTDEClient(g_ROBOT_IP, notifier_, resources_output_recipe_,
+                                                            actual_input_recipe)),
+               RTDEInvalidKeyException);
+}
+
+TEST_F(RTDEClientTest, recipe_comparison)
 {
   // Check that vectorized constructor provides same recipes as from file
   auto client = rtde_interface::RTDEClient(g_ROBOT_IP, notifier_, resources_output_recipe_, resources_input_recipe_);
@@ -284,14 +293,17 @@ TEST_F(RTDEClientTest, recipe_compairson)
   }
 }
 
-TEST_F(RTDEClientTest, get_data_package)
+TEST_F(RTDEClientTest, get_data_package_w_background_deprecated)
 {
   client_->init();
   client_->start();
 
   // Test that we can receive a package and extract data from the received package
   const std::chrono::milliseconds read_timeout{ 100 };
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wdeprecated-declarations"
   std::unique_ptr<rtde_interface::DataPackage> data_pkg = client_->getDataPackage(read_timeout);
+#pragma GCC diagnostic pop
   if (data_pkg == nullptr)
   {
     std::cout << "Failed to get data package from robot" << std::endl;
@@ -304,6 +316,234 @@ TEST_F(RTDEClientTest, get_data_package)
   client_->pause();
 }
 
+TEST_F(RTDEClientTest, get_data_package_w_background)
+{
+  client_->init();
+  client_->start();
+
+  // Test that we can receive a package and extract data from the received package
+  const std::chrono::milliseconds read_timeout{ 100 };
+
+  // Create an empty data package. Its timestamp should be 0.0
+  rtde_interface::DataPackage data_pkg(client_->getOutputRecipe());
+  double timestamp;
+  EXPECT_TRUE(data_pkg.getData("timestamp", timestamp));
+  ASSERT_TRUE(data_pkg.setData("timestamp", 0.0));
+
+  ASSERT_TRUE(client_->getDataPackage(data_pkg, read_timeout));
+
+  // Verify that we actually got data from the robot
+  EXPECT_TRUE(data_pkg.getData("timestamp", timestamp));
+  EXPECT_GT(timestamp, 0.0);
+
+  ASSERT_TRUE(client_->getDataPackage(data_pkg, read_timeout));
+  // Trying to get data with a very short timeout should fail
+  ASSERT_FALSE(client_->getDataPackage(data_pkg, std::chrono::milliseconds(1)));
+
+  // Check the second signature
+  auto data_pkg_ptr = std::make_unique<rtde_interface::DataPackage>(client_->getOutputRecipe());
+  ASSERT_TRUE(data_pkg_ptr->setData("timestamp", 0.0));
+  ASSERT_TRUE(client_->getDataPackage(data_pkg_ptr, std::chrono::milliseconds(100)));
+  EXPECT_TRUE(data_pkg_ptr->getData("timestamp", timestamp));
+  EXPECT_GT(timestamp, 0.0);
+
+  // Blocking call while packages are fetched in background should fail
+  ASSERT_FALSE(client_->getDataPackageBlocking(data_pkg_ptr));
+
+  // starting the background read twice should be fine (no effect)
+  ASSERT_NO_THROW(client_->startBackgroundRead());
+
+  ASSERT_NO_THROW(client_->stopBackgroundRead());
+  EXPECT_FALSE(client_->getDataPackage(data_pkg_ptr, std::chrono::milliseconds(100)));
+  ASSERT_NO_THROW(client_->startBackgroundRead());
+  EXPECT_TRUE(client_->getDataPackage(data_pkg_ptr, std::chrono::milliseconds(100)));
+
+  client_->pause();
+}
+
+TEST_F(RTDEClientTest, get_data_package_wo_background)
+{
+  client_->init();
+  client_->start(false);
+
+  auto data_pkg = std::make_unique<rtde_interface::DataPackage>(client_->getOutputRecipe());
+  ASSERT_TRUE(data_pkg->setData("timestamp", 0.0));
+  ASSERT_TRUE(client_->getDataPackageBlocking(data_pkg));
+
+  urcl::vector6d_t actual_q;
+  EXPECT_TRUE(data_pkg->getData("actual_q", actual_q));
+  double timestamp;
+  EXPECT_TRUE(data_pkg->getData("timestamp", timestamp));
+  EXPECT_GT(timestamp, 0.0);
+
+  // Non-blocking call should fail since we are not reading in background
+  ASSERT_FALSE(client_->getDataPackage(data_pkg, std::chrono::milliseconds(100)));
+  ASSERT_FALSE(client_->getDataPackage(*data_pkg, std::chrono::milliseconds(100)));
+
+  // We should be able to start background reading while the client is started without background
+  // reading and then query packages using the non-blocking call
+  client_->startBackgroundRead();
+  ASSERT_TRUE(client_->getDataPackage(data_pkg, std::chrono::milliseconds(100)));
+  double timestamp_2;
+  EXPECT_TRUE(data_pkg->getData("timestamp", timestamp_2));
+  EXPECT_GT(timestamp_2, timestamp);
+  EXPECT_FALSE(client_->getDataPackageBlocking(data_pkg));
+
+  client_->pause();
+}
+
+TEST_F(RTDEClientTest, get_data_package_fake_server)
+{
+  auto fake_rtde_server = std::make_unique<RTDEServer>(g_FAKE_RTDE_PORT);
+  // Skip the bootup check. If uptime is less then 40 seconds, data is read for one second to
+  // check for safety reset.
+  fake_rtde_server->setStartTime(std::chrono::steady_clock::now() - std::chrono::seconds(42));
+  client_.reset(new rtde_interface::RTDEClient("localhost", notifier_, resources_output_recipe_,
+                                               resources_input_recipe_, 100, false, g_FAKE_RTDE_PORT));
+  client_->init();
+  client_->start();
+
+  // Test that we can receive a package and extract data from the received package
+  const std::chrono::milliseconds read_timeout{ 100 };
+  auto data_pkg = rtde_interface::DataPackage(client_->getOutputRecipe());
+  if (!client_->getDataPackage(data_pkg, read_timeout))
+  {
+    std::cout << "Failed to get data package from robot" << std::endl;
+    GTEST_FAIL();
+  }
+
+  urcl::vector6d_t actual_q;
+  EXPECT_TRUE(data_pkg.getData("actual_q", actual_q));
+
+  URCL_LOG_INFO("Received data package from fake server: %s", data_pkg.toString().c_str());
+  client_.reset();
+}
+
+TEST_F(RTDEClientTest, reconnect_fake_server_background_read)
+{
+  auto fake_rtde_server = std::make_unique<RTDEServer>(g_FAKE_RTDE_PORT);
+  // Skip the bootup check. If uptime is less then 40 seconds, data is read for one second to
+  // check for safety reset.
+  fake_rtde_server->setStartTime(std::chrono::steady_clock::now() - std::chrono::seconds(42));
+  client_.reset(new rtde_interface::RTDEClient("localhost", notifier_, resources_output_recipe_,
+                                               resources_input_recipe_, 100, false, g_FAKE_RTDE_PORT));
+  client_->init(0, std::chrono::milliseconds(123), 3, std::chrono::milliseconds(100));
+  URCL_LOG_INFO("Client initiliazed");
+  client_->start();
+
+  std::atomic<bool> keep_running = true;
+  std::thread data_consumer_thread([this, &keep_running]() {
+    rtde_interface::DataPackage data_pkg(client_->getOutputRecipe());
+    const std::chrono::milliseconds read_timeout{ 100 };
+    while (keep_running)
+    {
+      if (client_->getDataPackage(data_pkg, read_timeout))
+      {
+        // URCL_LOG_INFO(data_pkg.toString().c_str());
+      }
+      else
+      {
+        std::this_thread::sleep_for(std::chrono::milliseconds(100));
+      }
+    }
+  });
+
+  std::this_thread::sleep_for(std::chrono::milliseconds(20));
+  fake_rtde_server.reset();
+  auto start_time = std::chrono::steady_clock::now();
+  while (std::chrono::steady_clock::now() - start_time < std::chrono::seconds(10) &&
+         client_->getClientState() != rtde_interface::ClientState::UNINITIALIZED)
+  {
+    std::this_thread::sleep_for(std::chrono::milliseconds(10));
+  }
+  ASSERT_EQ(client_->getClientState(), rtde_interface::ClientState::UNINITIALIZED);
+  URCL_LOG_INFO("Resetting rtde_server");
+  fake_rtde_server = std::make_unique<RTDEServer>(g_FAKE_RTDE_PORT);
+  fake_rtde_server->setStartTime(std::chrono::steady_clock::now() - std::chrono::seconds(52));
+
+  start_time = std::chrono::steady_clock::now();
+  while (std::chrono::steady_clock::now() - start_time < std::chrono::seconds(10) &&
+         client_->getClientState() != rtde_interface::ClientState::RUNNING)
+  {
+    std::this_thread::sleep_for(std::chrono::milliseconds(10));
+  }
+  ASSERT_EQ(client_->getClientState(), rtde_interface::ClientState::RUNNING);
+
+  if (data_consumer_thread.joinable())
+  {
+    keep_running = false;
+    data_consumer_thread.join();
+  }
+  rtde_interface::DataPackage data_pkg(client_->getOutputRecipe());
+  ASSERT_TRUE(client_->getDataPackage(data_pkg, std::chrono::milliseconds(100)));
+  URCL_LOG_INFO(data_pkg.toString().c_str());
+
+  client_.reset();
+  URCL_LOG_INFO("Done");
+}
+
+TEST_F(RTDEClientTest, reconnect_fake_server_blocking_read)
+{
+  auto fake_rtde_server = std::make_unique<RTDEServer>(g_FAKE_RTDE_PORT);
+  // Skip the bootup check. If uptime is less then 40 seconds, data is read for one second to
+  // check for safety reset.
+  fake_rtde_server->setStartTime(std::chrono::steady_clock::now() - std::chrono::seconds(42));
+  client_.reset(new rtde_interface::RTDEClient("localhost", notifier_, resources_output_recipe_,
+                                               resources_input_recipe_, 100, false, g_FAKE_RTDE_PORT));
+  client_->init(0, std::chrono::milliseconds(123), 3, std::chrono::milliseconds(100));
+  URCL_LOG_INFO("Client initiliazed");
+  client_->start(false);
+
+  std::atomic<bool> keep_running = true;
+  std::thread data_consumer_thread([this, &keep_running]() {
+    auto data_pkg = std::make_unique<rtde_interface::DataPackage>(client_->getOutputRecipe());
+    while (keep_running)
+    {
+      if (client_->getDataPackageBlocking(data_pkg))
+      {
+        URCL_LOG_INFO(data_pkg->toString().c_str());
+      }
+      else
+      {
+        std::this_thread::sleep_for(std::chrono::milliseconds(100));
+      }
+    }
+  });
+
+  std::this_thread::sleep_for(std::chrono::milliseconds(20));
+  fake_rtde_server.reset();
+  auto start_time = std::chrono::steady_clock::now();
+  while (std::chrono::steady_clock::now() - start_time < std::chrono::seconds(10) &&
+         client_->getClientState() != rtde_interface::ClientState::UNINITIALIZED)
+  {
+    std::this_thread::sleep_for(std::chrono::milliseconds(10));
+  }
+  ASSERT_EQ(client_->getClientState(), rtde_interface::ClientState::UNINITIALIZED);
+  URCL_LOG_INFO("Resetting rtde_server");
+  fake_rtde_server = std::make_unique<RTDEServer>(g_FAKE_RTDE_PORT);
+  fake_rtde_server->setStartTime(std::chrono::steady_clock::now() - std::chrono::seconds(52));
+
+  start_time = std::chrono::steady_clock::now();
+  while (std::chrono::steady_clock::now() - start_time < std::chrono::seconds(10) &&
+         client_->getClientState() != rtde_interface::ClientState::RUNNING)
+  {
+    std::this_thread::sleep_for(std::chrono::milliseconds(10));
+  }
+  ASSERT_EQ(client_->getClientState(), rtde_interface::ClientState::RUNNING);
+
+  if (data_consumer_thread.joinable())
+  {
+    keep_running = false;
+    data_consumer_thread.join();
+  }
+  auto data_pkg = std::make_unique<rtde_interface::DataPackage>(client_->getOutputRecipe());
+  ASSERT_TRUE(client_->getDataPackageBlocking(data_pkg));
+  URCL_LOG_INFO(data_pkg->toString().c_str());
+
+  client_.reset();
+  URCL_LOG_INFO("Done");
+}
+
 TEST_F(RTDEClientTest, write_rtde_data)
 {
   client_->init();
@@ -314,15 +554,12 @@ TEST_F(RTDEClientTest, write_rtde_data)
 
   // Make sure that the data has been written to the robot
   const std::chrono::milliseconds read_timeout{ 100 };
-  std::unique_ptr<rtde_interface::DataPackage> data_pkg = client_->getDataPackage(read_timeout);
-  if (data_pkg == nullptr)
-  {
-    std::cout << "Failed to get data package from robot" << std::endl;
-    GTEST_FAIL();
-  }
+
+  rtde_interface::DataPackage data_pkg(client_->getOutputRecipe());
+  ASSERT_TRUE(client_->getDataPackage(data_pkg, read_timeout));
 
   std::bitset<18> actual_dig_out_bits;
-  data_pkg->getData<uint64_t>("actual_digital_output_bits", actual_dig_out_bits);
+  data_pkg.getData<uint64_t>("actual_digital_output_bits", actual_dig_out_bits);
 
   // If we get the data package to soon the digital output might not have been updated, therefore we get the package a
   // couple of times
@@ -330,8 +567,8 @@ TEST_F(RTDEClientTest, write_rtde_data)
   int counter = 0;
   while (actual_dig_out_bits[0] != send_digital_output)
   {
-    data_pkg = client_->getDataPackage(read_timeout);
-    data_pkg->getData<uint64_t>("actual_digital_output_bits", actual_dig_out_bits);
+    ASSERT_TRUE(client_->getDataPackage(data_pkg, read_timeout));
+    data_pkg.getData<uint64_t>("actual_digital_output_bits", actual_dig_out_bits);
     if (counter == max_tries)
     {
       break;
@@ -364,11 +601,11 @@ TEST_F(RTDEClientTest, output_recipe_without_timestamp)
 
 TEST_F(RTDEClientTest, connect_non_running_robot)
 {
-  // We use an IP address on the integration_test's subnet
+  // Make sure that there's no simulator running exposing RTDE on localhost.
   client_.reset(
-      new rtde_interface::RTDEClient("192.168.56.123", notifier_, resources_output_recipe_, resources_input_recipe_));
+      new rtde_interface::RTDEClient("127.0.0.1", notifier_, resources_output_recipe_, resources_input_recipe_));
   auto start = std::chrono::system_clock::now();
-  EXPECT_THROW(client_->init(2, std::chrono::milliseconds(500), 1), UrException);
+  EXPECT_THROW(client_->init(2, std::chrono::milliseconds(50), 1), UrException);
   auto end = std::chrono::system_clock::now();
   auto elapsed = end - start;
   // This is only a rough estimate, obviously.
@@ -406,16 +643,12 @@ TEST_F(RTDEClientTest, check_all_rtde_output_variables_exist)
 
   // Test that we can receive and parse the timestamp from the received package to prove the setup was successful
   const std::chrono::milliseconds read_timeout{ 100 };
-  std::unique_ptr<rtde_interface::DataPackage> data_pkg = client_->getDataPackage(read_timeout);
-
-  if (data_pkg == nullptr)
-  {
-    std::cout << "Failed to get data package from robot" << std::endl;
-    GTEST_FAIL();
-  }
+  rtde_interface::DataPackage data_pkg(client_->getOutputRecipe());
+  ASSERT_TRUE(client_->getDataPackage(data_pkg, read_timeout));
 
   double timestamp;
-  EXPECT_TRUE(data_pkg->getData("timestamp", timestamp));
+  EXPECT_TRUE(data_pkg.getData("timestamp", timestamp));
+  EXPECT_GT(timestamp, 0.0);
 
   client_->pause();
 }
@@ -472,10 +705,27 @@ TEST_F(RTDEClientTest, check_unknown_rtde_output_variable)
   std::vector<std::string> incorrect_output_recipe = client_->getOutputRecipe();
   incorrect_output_recipe.push_back("unknown_rtde_variable");
 
-  client_.reset(new rtde_interface::RTDEClient(g_ROBOT_IP, notifier_, incorrect_output_recipe, resources_input_recipe_,
-                                               0.0, false));
+  // If unknown variables are not ignored, initialization should fail
+  EXPECT_THROW(client_.reset(new rtde_interface::RTDEClient(g_ROBOT_IP, notifier_, incorrect_output_recipe,
+                                                            resources_input_recipe_, 0.0, false)),
+               RTDEInvalidKeyException);
 
-  EXPECT_THROW(client_->init(), UrException);
+  // Unknown variables (by the control box) can be ignored, so initialization should succeed
+  if ((client_->getVersion().major == 5 && client_->getVersion().minor < 23) ||
+      (client_->getVersion().major == 10 && client_->getVersion().minor < 11))
+  {
+    std::vector<std::string> output_recipe = client_->getOutputRecipe();
+    output_recipe.push_back("actual_robot_energy_consumed");  // That has been added in 5.23.0 / 10.11.0
+    client_.reset(
+        new rtde_interface::RTDEClient(g_ROBOT_IP, notifier_, output_recipe, resources_input_recipe_, 0.0, true));
+    EXPECT_TRUE(client_->init());
+  }
+
+  // Passing a completely unknown variable should still lead to an exception, even if unknown
+  // variables are ignored.
+  EXPECT_THROW(client_.reset(new rtde_interface::RTDEClient(g_ROBOT_IP, notifier_, incorrect_output_recipe,
+                                                            resources_input_recipe_, 0.0, true)),
+               RTDEInvalidKeyException);
 }
 
 TEST_F(RTDEClientTest, empty_input_recipe)
@@ -487,16 +737,11 @@ TEST_F(RTDEClientTest, empty_input_recipe)
 
   // Test that we can receive and parse the timestamp from the received package to prove the setup was successful
   const std::chrono::milliseconds read_timeout{ 100 };
-  std::unique_ptr<rtde_interface::DataPackage> data_pkg = client_->getDataPackage(read_timeout);
-
-  if (data_pkg == nullptr)
-  {
-    std::cout << "Failed to get data package from robot" << std::endl;
-    GTEST_FAIL();
-  }
+  rtde_interface::DataPackage data_pkg(client_->getOutputRecipe());
+  ASSERT_TRUE(client_->getDataPackage(data_pkg, read_timeout));
 
   double timestamp;
-  EXPECT_TRUE(data_pkg->getData("timestamp", timestamp));
+  EXPECT_TRUE(data_pkg.getData("timestamp", timestamp));
 
   EXPECT_FALSE(client_->getWriter().sendStandardDigitalOutput(1, false));
 
@@ -506,20 +751,38 @@ TEST_F(RTDEClientTest, empty_input_recipe)
   client_->init();
   client_->start();
 
-  data_pkg = client_->getDataPackage(read_timeout);
-
-  if (data_pkg == nullptr)
-  {
-    std::cout << "Failed to get data package from robot" << std::endl;
-    GTEST_FAIL();
-  }
-  EXPECT_TRUE(data_pkg->getData("timestamp", timestamp));
+  ASSERT_TRUE(client_->getDataPackage(data_pkg, read_timeout));
+  EXPECT_TRUE(data_pkg.getData("timestamp", timestamp));
 
   EXPECT_FALSE(client_->getWriter().sendStandardDigitalOutput(1, false));
 
   client_->pause();
 }
 
+TEST_F(RTDEClientTest, test_initialization)
+{
+  // Test that initialization fails with 0 initialization attempts
+  EXPECT_THROW(client_->init(1, std::chrono::milliseconds(100), 0), UrException);
+
+  comm::TCPServer dummy_server(UR_RTDE_PORT);
+  dummy_server.start();
+
+  // Test that initialization fails when no RTDE interface is available on the robot
+  // within the given initialization attempts
+  // We use a dummy server here that doesn't implement the RTDE interface
+  // to simulate this scenario.
+  // The total time should be at least (initialization attempts - 1) * initialization timeout
+  // since the last attempt doesn't wait after failing.
+
+  URCL_LOG_INFO("Starting initialization timing test");
+  client_.reset(new rtde_interface::RTDEClient("127.0.0.1", notifier_, resources_output_recipe_, {}));
+  auto start = std::chrono::system_clock::now();
+  EXPECT_THROW(client_->init(2, std::chrono::milliseconds(10), 2, std::chrono::milliseconds(10)), UrException);
+  auto end = std::chrono::system_clock::now();
+  auto elapsed = end - start;
+  EXPECT_GE(std::chrono::duration_cast<std::chrono::milliseconds>(elapsed).count(), 20);
+}
+
 int main(int argc, char* argv[])
 {
   ::testing::InitGoogleTest(&argc, argv);
@@ -534,4 +797,4 @@ int main(int argc, char* argv[])
   }
 
   return RUN_ALL_TESTS();
-}
\ No newline at end of file
+}
